var express = require('express');
var cors = require('cors');
var favicon = require('serve-favicon');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var async = require('async');
var fs = require('fs');
var path = require('path');

// Expressアプリケーション作成
var app = express();

// 設定ファイル読み込み
var config = JSON.parse(fs.readFileSync('./config.json', 'utf8'));
var port = config.PORT;

var photo_dir = path.join(__dirname, config.PUBLIC, config.FILES);
var thumbnail_dir = path.join(photo_dir, 'thumbnail');
var tablename = config.TABLENAME;
var tableid = null;
var db = require('./lib/fusiontable');

// グローバル
global.BASEDIR = __dirname;
global.CONFIG = config;
global.DB = db;
global.PHOTO_DIR = photo_dir;
global.THUMBNAIL_DIR = thumbnail_dir;
global.TABLE_INFO = { tablename: tablename, tableid: tableid };

// テンプレート・エンジン設定
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'pug');

// CORS有効化
app.use(cors())

// favicon設定
app.use(favicon(path.join(__dirname, config.PUBLIC, 'favicon.ico')));

// リクエストBody
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));

// Cookie対応
app.use(cookieParser());

// 静的ファイルの設定
app.use(express.static(path.join(__dirname, config.PUBLIC)));

// ルーティング設定
app.use('/map', require('./routes/map'));
app.use('/files', require('./routes/files'));
app.use('/upload', require('./routes/upload'));

// 認証処理とサーバ起動
async.waterfall([
	function (callback) {
		db.get_token(function (err) {
			if (err === null) {
				db.get_tableid(tablename, function (id) {
					tableid = id;
					global.TABLE_INFO.tableid = tableid;
					callback(null)
				});
			}
			else {
				callback(err);
			}
		});
	}],
	function (err, res) {
		// サーバ起動
		console.log('photo_dir = ' + global.PHOTO_DIR);
		console.log('thumbnail_dir = ' + global.THUMBNAIL_DIR);
		console.log('tablename = ' + global.TABLE_INFO.tablename);
		console.log('tableid = ' + global.TABLE_INFO.tableid);
		app.listen(port, function () {
			console.log('Map Server app listening on port ' + port);
		});
	});



