var express = require('express');
var router = express.Router();
var fs = require('fs');
var path = require('path');
var async = require('async');
var exif = require('../lib/exif');

// コンフィグ
var photo_dir = global.PHOTO_DIR;
var thumbnail_dir = global.THUMBNAIL_DIR;
var myfiles = global.CONFIG.FILES;
var db = global.DB;

//  ファイル削除処理
function delete_file(name, cb) {
	let table_id = global.TABLE_INFO.tableid;
	db.get_by_name(table_id, name, function (res) {
		// rowid取得
		if (res) {
			let col = res[0];
			let row = res[1];
			let rowid = row[col.indexOf('rowid')];
			let file = row[col.indexOf('file')];
			let file_path = path.join(photo_dir, file);
			let thum_file_path = path.join(thumbnail_dir, file);
			// ファイル削除
			fs.unlink(file_path, function (err) {
				fs.unlink(thum_file_path, function (err) {
					// テーブル削除
					db.delete_by_id(table_id, rowid, function (err) {
						cb(null);
					});
				});
			});
		}
		else {
			cb(null)
		}
	});

}

// アルバム表示用のエントリ
router.get('/', function (req, res, next) {
	// ファイル一覧取得
	fs.readdir(photo_dir, function (err, files) {
		let file_list = [];
		if (err === null) {
			async.forEachSeries(files, function (val, cb) {
				if (!val.startsWith('.') && !val.startsWith('thumbnail')) {
					/* 撮影位置情報と名前をFusionTablesからら取得する。
					   取れなければ、ファイルのEXIFから取り出す。
					   今回は、１エントリづつ取り出しているが、性能のためにはまとめて取り出し、
					   キャッシュした方が良いが、今回は割愛。 */
					let table_id = global.TABLE_INFO.tableid;
					db.get_by_file(table_id, val, function (res) {
						if (res) {
							let col = res[0];
							let row = res[1];
							let name = row[col.indexOf('name')];
							let latlng = row[col.indexOf('Location')];
							// ファイルリストに追加
							file_list.push([val, name, latlng]);
							cb();
						}
						else {
							// 画像から撮影位置を取得
							exif.get_latlng(path.join(photo_dir, val), function (err, pos) {
								// ファイルリストに追加
								file_list.push([val, val, pos]);
								cb();
							});
						}
					});
				}
				else {
					cb();
				}
			}, function () {
				// 結果表示
				res.render('files', { title: 'アルバム', myfiles: myfiles, images: file_list });
			});
		}
		else {
			// 結果表示
			res.render('files', { title: 'アルバム', myfiles: myfiles, images: file_list });
		}
	});

});

// 削除処理用のエントリポイント
router.get('/delete', function (req, res, next) {
	let name = req.query.name;
	delete_file(name, function (err) {
		// 結果表示
		res.render('delete_result', { title: 'ファイル削除', file: name });
	});
});


module.exports = router;
