var express = require('express');
var router = express.Router();
var multer = require('multer');
var fs = require('fs');
var path = require('path');
var async = require('async');

var config = global.CONFIG;
var photo_dir = global.PHOTO_DIR;
var thumbnail_dir = global.THUMBNAIL_DIR;
var db = global.DB;

var mapsapi = require('@google/maps').createClient({
	key: config.APIKEY
});

// アップロードファイルの保存先設定
var upload = multer({ dest: photo_dir });

// アップロード画面のエンドポイント
router.get('/', function (req, res, next) {
	res.render('upload', { title: '写真アップロード', info: 'Upload View' });
});

var Exif = require('exif');
var ExifImage = Exif.ExifImage;
var exif_header_offset = 12;

function delete_old(file, cb) {
	let file_path = path.join(photo_dir, file);
	let thum_file_path = path.join(thumbnail_dir, file);
	fs.unlink(file_path, function (err) {
		fs.unlink(thum_file_path, function (err) {
			cb(null);
		});
	});
}

// アップロード処理エントリポイント
router.post('/', upload.single('pic'), function (req, res, next) {
	// ファイル名
	let file_name = req.file.originalname;
	let saved_file = path.join(photo_dir, req.file.filename);
	let table_id = global.TABLE_INFO.tableid;

	async.waterfall([
		// Exif
		function (callback) {
			try {
				new ExifImage({ image: saved_file }, function (error, exifData) {
					if (error) {
						console.log('Error: ' + error.message);
					}
					else {
						let orientation = exifData.image.Orientation;
						let lat_r = exifData.gps.GPSLatitudeRef;
						let latitude = 0;
						if (exifData.gps.GPSLatitude) {
							latitude = exifData.gps.GPSLatitude[0] + exifData.gps.GPSLatitude[1] / 60.0 + exifData.gps.GPSLatitude[2] / 3600.0;
						}
						if (lat_r != "N") {
							latitude = 0 - latitude;
						}
						let lon_r = exifData.gps.GPSLongitudeRef;
						let longtitude = 0;
						if (exifData.gps.GPSLongitude) {
							longtitude = exifData.gps.GPSLongitude[0] + exifData.gps.GPSLongitude[1] / 60.0 + exifData.gps.GPSLongitude[2] / 3600.0;
						}
						if (lon_r != "E") {
							longtitude = 0 - longtitude;
						}
						let thum_off = exifData.thumbnail.ThumbnailOffset;
						let thum_len = exifData.thumbnail.ThumbnailLength;
						let thum = null;
						// サムネイル画像を取り出す
						if (thum_len > 0) {
							let fd = fs.openSync(saved_file, 'r');
							let buffer = Buffer.alloc(thum_len);
							let num = fs.readSync(fd, buffer, 0, thum_len, thum_off + exif_header_offset);
							let file_path = path.join(thumbnail_dir, req.file.filename);
							let fdw = fs.openSync(file_path, 'w');
							fs.writeSync(fdw, buffer, 0, num);
							fs.closeSync(fd);
							fs.closeSync(fdw);
							if (num == thum_len) {
								thum = file_path;
							}
						}
						else {
							/* サムネイル画像がない場合は、元ファイルをサムネイ画像とする。
							   本当は、ここでサムネイル画像となるように縮小するが、今回は割愛。 */
							let to_file_path = path.join(thumbnail_dir, req.file.filename);
							fs.createReadStream(saved_file).pipe(fs.createWriteStream(to_file_path));
						}
						callback(null, latitude + "," + longtitude, thum);

					}
				});
			}
			catch (error) {
				console.log('Error ' + error.message);
				callback(new Error(error.message), null);
			}
		},
		function (latlng, thumb, callback) {
			// Google Maps APIで地理座標から住所に変換
			mapsapi.reverseGeocode({
				latlng: latlng, language: 'ja'
			}, function (err, response) {
				let address = latlng;
				if (!err) {
					address = response.json.results[0].formatted_address;
					console.log("address = " + address);
				}
				callback(null, latlng, thumb, address);
			});
		},
		// 
		function (latlng, thumb, address, callback) {
			async.waterfall([
				function (cb) {
					let rec = {};
					// file_name で存在確認
					db.get_by_name(table_id, file_name, function (res) {
						rec = { name: file_name, file: req.file.filename, Location: latlng };
						if (res) {
							// 存在していたら、元のファイルを削除してレコードをアップデートする
							let col = res[0];
							let row = res[1];
							let to_file = row[col.indexOf('file')];
							// set rowid
							rec['rowid'] = row[col.indexOf('rowid')];
							// ファイル削除
							delete_old(to_file, function (err) {
								cb(null, rec);
							});
						}
						else {
							cb(null, rec);
						}
					});

				},
				function (rec, cb) {
					// Fusion Tablesに登録する
					if (rec.rowid) {
						db.update(table_id, rec.rowid, rec, function (err) {
							if (err) {
								console.log("update error:" + err);
							}
							cb(null, rec);
						});
					}
					else {
						db.put(table_id, rec, function (err) {
							if (err) {
								console.log("put error:" + err);
							}
							cb(null, rec);
						});
					}
				}],
				function (err, res) {
					callback(null, latlng, address);
				}
			);
		}],
		function (err, latlng, address) {
			if (err) {
				console.log(err);
			}
			console.log(latlng + ":" + address);
			// 撮影場所をマップのマーカー表示するためのGoogle Maps API呼び出しを準備
			let mapview = "https://maps.googleapis.com/maps/api/staticmap?markers=" + latlng + "&zoom=16&size=400x400&key=" + config.APIKEY;
			res.render('upload_result', { file: file_name, info: "画像位置情報", sview: mapview, latlng: latlng, address: address });
		}
	);
});

module.exports = router;
